@@ -1,12 +1,38 @@
 require "mkmf"
+require "rubygems" unless defined?(Gem)
 
-gem_narray_path=File::dirname(`gem which narray`)
-narray_include=gem_narray_path
-narray_lib=gem_narray_path
-dir_config('narray',narray_include,narray_lib)
- 
 dir_config('fftw3','/usr/local')
 
+require "rbconfig"
+so = RbConfig::CONFIG["DLEXT"]
+
+raise("Your gem is too old") unless Gem.respond_to?(:find_files) 
+
+env = ENV['NARRAY_TYPE']
+if env == "NArray" || env == "narray"
+  na_type = "narray"
+elsif env == "NumRu::NArray" || env == "numru-narray"
+  na_type = "numru-narray"
+elsif !env.nil?
+  raise "Unsupported value for the environment variable NARRAY_TYPE: #{env}"
+else
+  if Gem.find_files("narray.h").length > 0
+    na_type = "narray"
+  elsif Gem.find_files("numru/narray/narray.h").length > 0
+    na_type = "numru-narray"
+  end
+end
+
+case na_type
+when "narray"
+  narray_include = File.expand_path(File.dirname(Gem.find_files("narray.h")[0]))
+  narray_lib = File.expand_path(File.dirname(Gem.find_files("narray." + so)[0]))
+when "numru-narray"
+  narray_include = File.expand_path(File.dirname(Gem.find_files("numru/narray/narray.h")[0]))
+  narray_lib = File.expand_path(File.dirname(Gem.find_files("numru/narray/narray." + so)[0]))
+end
+
+dir_config('narray', narray_include, narray_lib)
 if ( ! ( have_header("narray.h") && have_header("narray_config.h") ) ) then
    print <<-EOS
    ** configure error **  
@@ -39,8 +65,4 @@ if have_library("fftw3f")
   $CFLAGS += ' -DFFTW3_HAS_SINGLE_SUPPORT'
 end
 
-if /cygwin|mingw/ =~ RUBY_PLATFORM
-   have_library("narray") || raise("ERROR: narray library is not found")
-end
-
 create_makefile("numru/fftw3/fftw3")
